import logging
import os
import shlex
import random
import re
import subprocess
from multiprocessing.pool import ThreadPool
from typing import List, Set, Tuple, Union

import tqdm

from .helpers import SpecialCharOperation


class TestCase():
    def __init__(self, command: List[str], char_offset: int, arg_index: int, scan_range: List[int], pre_command: List[str], post_command: List[str], exit_code_only: bool, timeout: int):
        self.command = command
        self.char_offset = char_offset
        self.arg_index = arg_index
        self.scan_range = scan_range
        self.pre_command = pre_command
        self.post_command = post_command
        self.exit_code_only = exit_code_only
        self.timeout = timeout


class TestProcessObfuscation():
    def __init__(self, test_case: TestCase, threads: int, log: logging.Logger):
        self.log = log
        self.command = test_case.command
        self.command_flat = ' '.join(test_case.command)
        self.pre_command = test_case.pre_command
        self.post_command = test_case.post_command
        self.expected_code, self.expected_output = self.__get_expected_result__()
        self.exit_code_only = test_case.exit_code_only
        self.scan_range = test_case.scan_range
        self.timeout = test_case.timeout
        self.threads = threads
        self.tqdm = None

    # Class Methods
    @classmethod
    def __randomise__(cls, command_parts: str) -> str:
        randomised = ''.join(random.choices('0123456789ABCDEF', k=10))
        return command_parts.replace('{random}', randomised)

    @classmethod
    def select_arg_index(cls, command: List[str], arg_index: Union[int, None]) -> int:
        # Iterate over command-line arguments
        for i, command_part in enumerate(command):
            # The first argument will be the process, so ignore
            if i == 0 or (arg_index is not None and i != arg_index):
                continue
            # Check if the first character is a slash or hyphen - if so, select it
            if command_part[0] in ['/', '-']:
                return i
        # If no option char was found, fall back to the first argument
        #  (least likely to be something like a filename)
        return 1

    @classmethod
    def get_command_part_offset(cls, command_part: str) -> int:
        # We prefer the below chars, because they have 'equivalents' in the Spacing Modifier Letters range (0x02B0 - 0x02FF)
        preferred_chars = 'hjrwyxsl'
        # Iterate over each char in the given command-line optoin
        for i, char in enumerate(command_part.lower()):
            # Ignore the first one - it will either be an option char, or
            #  the first char of a keyword which is usually not the best candidate
            if i == 0:
                continue
            if any([char == preferred_char for preferred_char in preferred_chars]):
                return i+1

        return (2 if command_part[1] != '-' else 3) if len(command_part) > 1 else 1

    def __get_expected_result__(self) -> Tuple[int, str]:
        # Prepare command to run
        command = self.__randomise__(self.command_flat)
        # Run 'normal' command to get expected exit code
        try:
            result = self.__execute_command__(command, pre_command=self.pre_command)
            exit_code, stdout = result.returncode, "{} / {}".format(result.stdout, result.stderr)
            # Check if observed exit code is 0
            if exit_code != 0:
                self.log.warning("Observed exit code is {}, which is not 0 as usual".format(exit_code))
                self.log.warning("Test outcome may contain unexpected results")
                self.log.warning(stdout)
                #sys.exit(-1)
        except FileNotFoundError:
            self.log.error("Command \"{}\" could not be executed: file not found".format(command))
            raise
        return exit_code, stdout

    # Private Methods
    def __test_commands__(self, test: str, commands: List[Tuple[int, str]]) -> Set[Tuple[int, str]]:
        self.log.info('Preparing {} commands to run'.format(len(commands)))
        # Prepare ThreadPool
        with ThreadPool(self.threads) as thread_pool:
            # Prepare progress bar
            with tqdm.tqdm(total=len(commands), desc=test, position=1, leave=False) as self.tqdm:
                # Submit all commands to ThreadPool, collect results
                outcomes = list(thread_pool.map(self.__run_command__, [command for _, command in commands]))
        # Return all results when output was True
        return set([(identifier, command) for outcome, (identifier, command) in zip(outcomes, commands) if outcome])

    def __run_command__(self, command: str):
        # Prepare command
        command = self.__randomise__(command)
        #self.log.info('About to run command "{}"'.format(shlex.split(command, posix=False)))
        try:
            # Run command
            result = self.__execute_command__(command, timeout=self.timeout, pre_command=self.pre_command)
            exit_code, stdout = result.returncode, "{} / {}".format(result.stdout, result.stderr)
            self.log.info('Exit code {} observed ({} desired) for {}'.format(exit_code, self.expected_code, command))
            # Return result
            return exit_code == self.expected_code and (self.exit_code_only or stdout == self.expected_output)
        except subprocess.TimeoutExpired:
            self.log.warning('Timeout ({}s) elapsed for command "{}"'.format(self.timeout, command))
            return False
        except Exception as e:
            self.log.info("Exception when executing \"{}\": {}".format(command, e))
            return False
        finally:
            if self.post_command:
                result = None
                try:
                    result = self.__execute_command__(self.post_command)
                except Exception as e:
                    self.log.warning("Post command caused exception ({})".format(e))
                finally:
                    self.log.info("Post command exited with exit code {}".format(result.returncode if result else "?"))
            if self.tqdm:
                self.tqdm.update()

    def __get_option_argument__(self, arg_index: int) -> int:
        return self.select_arg_index(self.command, arg_index)

    def __execute_command__(self, command: str, timeout: int = None, pre_command: list[str] = []) -> subprocess.CompletedProcess:
        cmd = command.split(' ') #list(shlex.shlex(command, punctuation_chars=True, posix=True)) #[shlex.quote(c) for c in shlex.split(command, posix=True)]
        #self.log.debug('About to run command "{}"'.format(' '.join(cmd)))
        p1 = subprocess.Popen(shlex.split(pre_command, posix=True) if os.sep == '/' else pre_command, stdin=subprocess.PIPE, stdout=subprocess.PIPE) if pre_command else None
        return subprocess.run(cmd if os.sep == '/' else command, stdin=p1.stdout if p1 else None, capture_output=True, timeout=timeout)

    # Public Methods
    def check_special_chars(self, char_at_position: int, operation: SpecialCharOperation) -> Set[Tuple[int, str]]:
        # Prepare operation
        op = None
        if operation == SpecialCharOperation.INSERT:
            op = "insert"
            command_parts = self.command_flat[:char_at_position+1], self.command_flat[char_at_position+1:]
        elif operation == SpecialCharOperation.REPLACE:
            op = "replace"
            command_parts = self.command_flat[:char_at_position], self.command_flat[char_at_position+1:]
        else:
            raise ValueError('Unexpected operation {}'.format(operation))
        self.log.info("Starting 'special chars ({})' test".format(op))
        # Prepare commands to run (excluding 'original' command)
        new_commands = [(ordinal, chr(ordinal).join(command_parts)) for ordinal in self.scan_range if operation != SpecialCharOperation.REPLACE or chr(ordinal).lower() != self.command_flat[char_at_position].lower()]
        # Run commands, return results
        return self.__test_commands__("Special chars ({})".format(op), new_commands)

    def check_quote_injection(self, arg_index: int) -> Union[str, None]:
        self.log.info("Starting 'quote insertion' test")
        # Check if valid command was given
        if not self.command or len(self.command) < 2:
            return None
        # Simply add quotes between 0,1 and 1,2 of first argument - e.g. net s"t"art
        command_part = self.__get_option_argument__(arg_index)
        quote = '"' # if os.sep != '/' else '\\"'
        test_arg = self.command[command_part][:1] + quote + self.command[command_part][1] + quote + self.command[command_part][2:]
        new_command = ' '.join(self.command[:command_part] + [test_arg] + self.command[command_part+1:])
        # Quotes don't work in subprocess when using lists
        result = self.__run_command__(new_command)
        # Return command if working, blank string if not
        return new_command if result else ''

    def check_shortened_option(self, arg_index: int) -> Union[None, Set[Tuple[int, str]]]:
        self.log.info("Starting 'shortened option' test")
        # Check if valid command was given
        if not self.command or len(self.command) < 2:
            raise ValueError('Unexpected length {}'.format(len(self.command)))
        # Shorten the second argument by [1..len(command)-1] chars
        selected_command_part_index = self.__get_option_argument__(arg_index)
        selected_command_part = self.command[selected_command_part_index]
        new_commands = []
        # check if there are any colons or equal signs in there (e.g. /active:true)
        selected_command_part_split = list(filter(None, re.split("[:=]", selected_command_part)))
        if len(selected_command_part_split) > 1:
            # If so, only try to shorten the first bit - leave the argument intact (e.g. /activ:true.../a:true)
            size = len(selected_command_part_split[0])
            # Check if argument is long enough
            if size <= 2:
                self.log.info("Length of selected command-line option is {}, cannot be further shortened".format(size))
                return None
            for i in range(size-1):
                test_arg = selected_command_part[0:(i+1)] + selected_command_part[size:]
                new_commands.append((i, ' '.join(self.command[:selected_command_part_index] + [test_arg] + self.command[(selected_command_part_index+1):])))
        else:
            # If not, just shorten the full argument
            size = len(selected_command_part)
            # Check if argument is long enough
            if size <= 2:
                self.log.info("Length of selected command-line option is {}, cannot be further shortened".format(size))
                return None
            for i in range(size-1):
                test_arg = selected_command_part[0:(i+1)]
                new_commands.append((i, ' '.join(self.command[:selected_command_part_index] + [test_arg] + self.command[(selected_command_part_index+1):])))
        # Return results of commands if any were generated, None if not
        return self.__test_commands__("Shorthand command", new_commands) if new_commands else None

    def check_option_char(self, arg_index_start: int = 1) -> Union[None, Set[Tuple[int, str]]]:
        self.log.info("Starting 'option char subtitution' test")

        delimiters_alternative = ['/', '\\', '\u2215', '\u244a', '\u2044', '\u29F8', '\u002D', '\u007E', '\u00AD', '\u058A', '\u05BE', '\u1400', '\u1806', '\u2010', '\u2010', '\u2012', '\u2013', '\u2014', '\u2015', '\u2053', '\u2212', '\u2212', '\u2212', '\u2E17', '\u2E3A', '\u2E3B', '\u301C', '\u3030', '\u30A0', '\uFE31', '\uFE32', '\uFE58', '\uFE63', '\uFF0D']
        delimiters_standard = ['/', '-']
        self.log.info("Looking for arguments starting with one of: {}".format(' '.join(delimiters_standard)))

        # Iterate over command-line arguments
        for i, command in enumerate(self.command[arg_index_start:], start=arg_index_start):
            # Check if delimiter is present
            if any([command.startswith(delimiter) for delimiter in delimiters_standard]):
                # If so, find its position
                pos = sum([len(c)+1 for c in self.command[:i]])
                # Break command in two parts
                command_parts = self.command_flat[:pos], self.command_flat[pos+1:]
                self.log.info("Argument found ({})".format(command))
                self.log.info("Preparing {} alternative signs to test".format(len(delimiters_alternative)))
                break
        else:
            self.log.info("No arguments found")
            return None  # No option char found

        # Prepare commands to test
        new_commands = [(ord(char), char.join(command_parts)) for char in delimiters_alternative]
        return self.__test_commands__("Dash/hyphen", [command for command in new_commands if command[1] != self.command_flat])



import datetime
import os
import re
from typing import Dict, List, Set, Tuple, Union

import colorama
import terminaltables

from .helpers import ranges

colorama.init()


def print_results(results: Dict[str, Dict[str, Union[Set[Tuple[int, str]], str, None]]]) -> None:
    """ For a dict with (test, result_dict), prepare a terminaltables AsciiTable and print to stdout """
    matrix = [['Process', 'Dash/Hyphen', 'Char (insert)', 'Char (replace)', 'Quotes', 'Shortened']]
    for display_name, test_outcomes in results.items():
        matrix.append([display_name] + [parse_outcome(test_outcome) for test_outcome in test_outcomes.values()])

    print(terminaltables.AsciiTable(matrix).table)


def parse_outcome(value: Union[Set[Tuple[int, str]], str, None]) -> str:
    """Prepare a terminal-printable output based on a given test outcome"""

    if value is None:  # If value is None, disregard
        return 'N/A'
    elif value:  # If value is positive, return Yes
        return_format = str(colorama.Back.GREEN) + '{}' + str(colorama.Style.RESET_ALL)
        if isinstance(value, str):  # If result is a string, only output Yes
            return return_format.format('Yes')
        elif isinstance(value, set):  # If result is a list, provide a count as well
            return return_format.format('Yes ({})'.format(len(value)))
        else:
            raise Exception("Unknown result type {}".format(type(value)))
    else:  # Value is negative, return No
        return str(colorama.Back.RED) + 'No' + str(colorama.Style.RESET_ALL)


def write_report(report_dir: str, name: str, command: str, char_offset: int, scan_range: List[int], test_outcomes: Dict[str, Union[Set[Tuple[int, str]], str, None]]) -> None:
    """Write a report, containing test results, to a given directory"""

    # Prepare output path
    file_name = '{}.log'.format(re.sub(r'[\\/*?:"<>|]', "", name))
    output_file = os.path.join(report_dir, file_name)

    # Open file
    with open(output_file, 'w', encoding='utf-16le') as f:
        # Print header
        f.write('PROCESS OBFUSCATION REPORT FOR {}\n'.format(name))
        f.write('- Generated on {}\n'.format(datetime.datetime.now().isoformat()))
        f.write('- Command used      : {}\n'.format(command))
        f.write('- Insertion position: {}\n'.format((' '*char_offset)+'^'))
        f.write('- Char ranges scanned: {}\n'.format(ranges(scan_range)))

        # Dash/hypten test
        if 'option_char' in test_outcomes:
            f.write('\n:: Option Char Substitution\n')
            outcomes = test_outcomes['option_char']
            if outcomes is not None:
                if outcomes:
                    f.write('The following {} commands were found to be working:\n'.format(len(outcomes)))
                    for identifier, command in sorted(outcomes, key=lambda x: x[0]):
                        f.write('0x{:0>4X} : {}\n'.format(identifier, command))
                else:
                    f.write('No alternative commands were found.\n')
            else:
                f.write('The command does not contain any arguments starting with a slash or dash.\n')

        # Character insertion test
        outcomes = test_outcomes.get('char_insert')
        if outcomes is not None:
            f.write('\n:: Character Insertion\n')
            if outcomes:
                f.write('The following {} commands were found to be working:\n'.format(len(outcomes)))
                for identifier, command in sorted(outcomes, key=lambda x: x[0]):
                    try:
                        f.write('0x{:0>4X} : {}\n'.format(identifier, command))
                    except:
                        f.write('0x{:0>4X} : (character can not be printed)\n'.format(identifier))
            else:
                f.write('No alternative commands were found.\n')

        # Character substitution test
        outcomes = test_outcomes.get('char_substitution')
        if outcomes is not None:
            f.write('\n:: Character Substitution\n')
            if outcomes:
                f.write('The following commands were found to be working:\n')
                for identifier, command in sorted(outcomes, key=lambda x: x[0]):
                    try:
                        f.write('0x{:0>4X} : {}\n'.format(identifier, command))
                    except:
                        f.write('0x{:0>4X} : (character can not be printed)\n'.format(identifier))
            else:
                f.write('No alternative commands were found.\n')

        # Quote insertion test
        outcome = test_outcomes.get('quotes')
        if outcome is not None:
            f.write('\n:: Quote Insertion\n')
            if outcome:
                f.write('Inserting quotes in the first argument did work, such as:\n')
                f.write('{}\n'.format(outcome))
            else:
                f.write('Inserting quotes in the first argument did not appear to be working.\n')

        # Shorthand commands
        outcomes = test_outcomes.get('shorthand_command')
        f.write('\n:: Shorthand Commands\n')
        if outcomes is not None:
            if outcomes:
                f.write('The following {} commands were found to be working:\n'.format(len(outcomes)))
                for identifier, command in sorted(outcomes, key=lambda x: len(x[1])):
                    f.write('{}\n'.format(command))
            else:
                f.write('No alternative commands were found.\n')
        else:
            f.write('The command is too short to be further shortened.\n')



import enum
import itertools
import logging
from typing import List

import tqdm


class SpecialCharOperation(enum.Enum):
    INSERT = 1
    REPLACE = 2


class TqdmHandler(logging.Handler):
    def emit(self, record: logging.LogRecord):
        try:
            msg = self.format(record)
            tqdm.tqdm.write(msg)
            self.flush()
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)


def ranges(i: List[int]) -> str:
    """Turns a list of integers to a string representation of the contained ranges"""
    result = []
    for _, b in itertools.groupby(enumerate(i), lambda pair: pair[1] - pair[0]):
        b = list(b)
        result.append((b[0][1], b[-1][1]))

    return ' '.join(['0x{:0>4X}..0x{:0>4X}'.format(x, y) for x, y in result])



import argparse
import json
import logging
import os
import re
import shlex
from argparse import Namespace
from typing import List, Tuple

import tqdm

from .helpers import SpecialCharOperation, TqdmHandler
from .output_results import print_results, write_report
from .test_process_obfuscation import TestCase, TestProcessObfuscation


def run_tests(test_cases: List[TestCase], threads: int, report_dir: str, log: logging.Logger) -> None:
    """ For a list of commands: run tests, create reports, write summary to stdout """
    result = {}
    with tqdm.tqdm(test_cases, position=0, disable=len(test_cases) <= 1) as pbar:
        for test_case in pbar:
            # Extract display name
            _, display_name = os.path.split(test_case.command[0].strip('"'))
            pbar.set_description('Testing {}'.format(display_name))

            try:
                # Create test class
                test = TestProcessObfuscation(test_case, threads, log)

                # Run individual tests
                test_outcomes = {'option_char': test.check_option_char(test_case.arg_index),
                                 'char_insert': test.check_special_chars(test_case.char_offset, SpecialCharOperation.INSERT),
                                 'char_substitution': test.check_special_chars(test_case.char_offset, SpecialCharOperation.REPLACE),
                                 'quotes': test.check_quote_injection(test_case.arg_index),
                                 'shorthand_command': test.check_shortened_option(test_case.arg_index)
                                 }

                # Parse results
                result[display_name] = test_outcomes

                # Write report
                write_report(report_dir, display_name, ' '.join(test_case.command), test_case.char_offset, test_case.scan_range, test_outcomes)
            except Exception:
                log.error("Unexpected error when executing", exc_info=True)

    # Print results to stdout
    print_results(result)


def prepare_command(parser: argparse.ArgumentParser, command_flat: str, char_offset: int) -> Tuple[List[str], int, int]:
    """ Turns a flattened command string into a list, and generates a character offset in case none was provided """
    log.info("Preparing parameters for {}".format(command_flat))
    command_list = shlex.split(command_flat, posix=os.sep=='/')
    arg_index = -1
    if char_offset is not None:
        if not (0 <= char_offset and char_offset < len(command_flat)):
            parser.error("char offset not within bounds of (0 .. command length)")
        else:
            selected_char = command_flat[char_offset]
            if (0 <= char_offset and char_offset < len(command_list[0])):
                parser.error("cannot add characters to process name")
            if not re.findall(r'[a-zA-Z0-9]', selected_char):
                parser.error("selected char '{}' is not alphanumeric".format(selected_char))

            i = 0
            for arg_index, arg in enumerate(command_list):
                i += len(arg)
                if i > char_offset:
                    break
    else:
        # Second char of first argument (not counting process name)
        arg_index = TestProcessObfuscation.select_arg_index(command_list, None)
        command_part_offset = TestProcessObfuscation.get_command_part_offset(command_list[arg_index].split(':')[0])

        char_offset = len(' '.join(command_list[0:arg_index])) + command_part_offset


        log.info("No char offset specified - using second char of first argument instead")
    log.info("Char offset = {} (char '{}', argument index = {})".format(char_offset, command_flat[char_offset], arg_index))
    return (command_list, char_offset, arg_index)


def create_test_case(args: Namespace, parser: argparse.ArgumentParser, json_file: bool) -> TestCase:
    """Generates a TestCase object based on the given arguments (either on the command line or in the JSON file)"""

    command = None
    if args.command:
        command, char_offset, arg_index = prepare_command(parser, args.command, args.char_offset)
    else:
        raise ValueError("'--command' is required" if not json_file else "'command' should be specified")

    # Parse char scan range options
    scan_range = None

    if args.custom_range:
        if (args.range is None or args.range == 'custom'):
            scan_range = sum(args.custom_range, [])
            if 0 in scan_range:
                parser.error("null bytes can not be included in custom ranges. Try starting from 1 instead.")
        else:
            raise ValueError("--custom_range can not be set when --range is not set to 'custom'.")
    elif args.range is None or args.range == 'educated':
        scan_range = list(range(0x01, 0xFF)) + list(range(0x0100, 0x052F)) + list(range(0x2070, 0x218F)) + list(range(0xFF00, 0xFFEF))
    elif args.range == 'full':
        scan_range = list(range(0x01, 0xFFFF))
    elif args.range == 'ascii':
        scan_range = list(range(0x01, 0xFF))
    else:
        raise ValueError("Unexpected range '{}'".format(args.range))

    log.info('{} range selected ({} values)'.format(args.range, len(scan_range)))

    # Parse report output dir
    log.info('Report files will be stored in {}'.format(os.path.abspath(args.report_dir)))
    if not os.path.isdir(args.report_dir):
        parser.error("path specified in --report_dir does not exist.")

    return TestCase(command=command, char_offset=char_offset, arg_index=arg_index, scan_range=scan_range, pre_command=args.pre_command, post_command=args.post_command, exit_code_only=args.exit_code_only, timeout=args.timeout)


# Prepare logger
log = logging.getLogger(__name__)
log.setLevel(logging.DEBUG)


def parse_arguments() -> None:
    # Prepare arguments
    parser = argparse.ArgumentParser(add_help=False, description='Tool for identifying executables that have command-line options that can be obfuscated.')

    parser._action_groups.pop()
    required = parser.add_argument_group('required arguments (either is required)')
    optional_c = parser.add_argument_group('optional --command arguments')
    optional = parser.add_argument_group('optional arguments')



    # Global settings
    optional.add_argument('--threads', metavar='n', type=int, help='Number of threads to use', default=None)
    optional.add_argument('--verbose', action='store_true', help='Increase output verbosity')
    optional.add_argument('--quiet', action='store_true', help='Decrease output verbosity')
    optional.add_argument('--report_dir', metavar='c:\\path\\to\\dir', type=str, help='Path to save report files to', default="."+os.sep)
    optional.add_argument('--log_file', metavar='c:\\path\\to\\file.log', type=str, help='Path to save log to')
    optional.add_argument('--help', action='help', default=argparse.SUPPRESS, help='Show this help message and exit')

    # Single command options
    required.add_argument('--command', metavar='"proc /arg1 /arg2"', type=str, help='Single command to test')
    optional_c.add_argument('--range', choices=['full', 'educated', 'ascii', 'custom'], help='Character range to scan (default=educated)')
    optional_c.add_argument('--custom_range', metavar='0x??..0x??', type=lambda x: list(range(int(x.split('..')[0], 0), int(x.split('..')[1], 0))), nargs='+', help='Range to scan')
    optional_c.add_argument('--char_offset', metavar='n', type=int, help='Character position used for insertion and replacement')
    optional_c.add_argument('--pre_command', metavar='process_name', type=str, help='Command to run unconditionally before each attempt, piping the result to the stdin')
    optional_c.add_argument('--post_command', metavar='process_name', type=str, help='Command to run unconditionally after each attempt (e.g. to clean up)')
    optional_c.add_argument('--exit_code_only', action='store_true', help='Only base success on the exit code (and not the output of the command)')
    optional_c.add_argument('--timeout', metavar='n', type=float, help='Number of seconds per execution before timing out.', default=2)

    # JSON File options
    required.add_argument('--json_file', metavar='c:\\path\\to\\file.jsonl', type=str, help='Path to JSON file (JSON Line formatted) containing commands config')

    args = parser.parse_args()

    tests = []

    # Parse verbosity options
    stream_handler = TqdmHandler()
    stream_handler.setFormatter(logging.Formatter("[%(levelname)-.4s] %(message)s"))
    if args.verbose:
        stream_handler.setLevel(logging.DEBUG)
    elif args.quiet:
        stream_handler.setLevel(logging.ERROR)
    else:
        stream_handler.setLevel(logging.WARNING)
    log.addHandler(stream_handler)

    # Parse log file options
    if args.log_file:
        file_path = os.path.abspath(args.log_file)
        file_handler = logging.FileHandler(file_path, 'w', encoding='utf-16le')
        file_handler.setLevel(logging.DEBUG)
        file_handler.setFormatter(logging.Formatter('%(asctime)s [%(levelname)-.4s] (%(threadName)-10s) %(message)s'))
        log.addHandler(file_handler)

    if args.json_file:
        single_command_args = ['command', 'range', 'custom_range', 'char_offset', 'pre_command', 'post_command', 'exit_code_only']
        given_args = {key: value for (key, value) in args._get_kwargs()}
        if any([given_args[x] for x in single_command_args]):
            raise ValueError("When --json_file is specified, the following arguments should not be specified on the command line but in the JSON file: {}".format(', '.join(single_command_args)))

        with open(args.json_file, 'rt') as f:
            for line in f.readlines():
                if not line.strip():
                    continue
                t_args = argparse.Namespace()
                t_args.__dict__.update(json.loads(line))
                args = parser.parse_args(namespace=t_args)
                tests.append(create_test_case(args, parser, True))
    else:
        tests = [create_test_case(args, parser, False)]

    run_tests(tests, threads=args.threads, report_dir=args.report_dir, log=log)


if __name__ == "__main__":
    parse_arguments()



import setuptools

description_short = "Project for identifying executables that have command-line options that can be obfuscated, possibly bypassing detection rules."
with open("README.md", "r", encoding="utf-8") as fh:
    description_long = fh.read()

setuptools.setup(
    name="analyse_obfuscation",
    version="1.1.0",
    author="@Wietze",
    description=description_short,
    long_description=description_long,
    long_description_content_type="text/markdown",
    url="https://github.com/wietze/windows-command-line-obfuscation",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: Microsoft :: Windows",
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'Topic :: Security',
        'Topic :: Internet :: Log Analysis',
        'Environment :: Console',
    ],
    entry_points={
        'console_scripts': [
            'analyse_obfuscation = analyse_obfuscation.run:parse_arguments'
        ],
    },
    install_requires=['tqdm', 'colorama', 'terminaltables']
)
